""" Text for the application."""

# Error messages
LOCATION_ERR = 'The config file is not in the expected location:'
DEFAULTS_ERR = 'No default config defined'
INVALID_TOML = 'Invalid toml format in'
INVALID_JSON = 'Invalid json format in'
CORRUPT_FILE = 'Corrupt config file.'
MISSING_ATTR = 'Missing attribute:'
NOT_OF_TYPE = 'not of type'
FIELD = 'Field'
NOT_IN_DICT = 'not in __dict__'
